#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <errno.h>

using namespace std;

extern int errno;

// This system demonstrates the use of the socket() message passing
// IPC mechanism.  Sockets are different from pipes and shared memory 
// because they can operate BETWEEN computers using the network.
//
// The purpose of the system is to provide a simple "chat" application.  
// The system consists of two programs "server" which sets up a TCP 
// port and accepts connections from clients.  And "client" which 
// connects to a TCP port to establish a connection with the server. 
// Both the client and server send messages back and forth in a 
// simplistic manner:
//
// SERVER:
// Loop until client types "/q":
//   Server: Wait for user to type in message
//   Server: Send message to client
//   Server: Wait for client to send message
//   Server: Display message from client
//
// CLIENT:
// Loop until "/q":
//   Client: Wait for server to send message
//   Client: Display message from server
//   Client: Wait for user to type in message
//   Client: Send message to client

int main(void)
{
  ////////////////////////////////////////////////////////////////////
  // Create a socket.  
  ////////////////////////////////////////////////////////////////////  
  int s = socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);

  if (s <= 0) {
    cerr << "clientr: Socket creation failed." << endl;
    exit(-1);
  }

  ////////////////////////////////////////////////////////////////////
  // Connect to server port
  ////////////////////////////////////////////////////////////////////
  sockaddr_in address;
  int addressSize = sizeof(address);
  bzero(&address,addressSize);
  address.sin_family=AF_INET;                         
  address.sin_port  = htons(8800); // <-------------------  port 15000
  if (!inet_aton("127.0.0.1",&address.sin_addr))  // <- IP 127.0.0.1 
  {
    cout << "inet_aton() failed" << endl;
  }
    
  //
  // This local buffer will be used to copy messages to/from socket.
  // Sockets require a contiguous block of memory to copy messages.
  // This could be an array of characters, integers, objects, whatever.
  //
  // The format of messages between a client and server is called a
  // PROTOCOL.  As a programmer, YOU have to decide upon a protocol.
  //
  // To keep things simple with this example, we've created an array
  // of characters.  Messages are transmitted as C-Strings.  A message
  // always starts at buffer[0] and always ends at the first position 
  // in the buffer where a '\0' is encountered.
  //
  const int BUFFER_SIZE = 100;
  char buffer[BUFFER_SIZE];

  do {


    // Get keyboard input from user
    cout << "client: Type in your message and press <ENTER>: ";
    cout.flush();
    cin.getline(buffer,BUFFER_SIZE);

    // Send message to server
    sendto(s,buffer,strlen(buffer),0,(struct sockaddr*) &address, (socklen_t) addressSize);

    // Stop when user types the string "\q"
  } while (strcmp(buffer,"/q"));

  // Cleanup
  close(s);

  return 1;
}
